/***************************************************************************
                          flmonview.cpp  -  description
                             -------------------
    begin                : Sat Mar 24 2001
    copyright            : (C) 2001 by Christian Traber
    email                : christian@traber-net.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "flmonview.h"

float NiCd[21] = {2.2,2.31, 2.355, 2.392, 2.412, 2.433, 2.449, 2.462, 2.472, 2.483, 2.491,
	2.498, 2.503, 2.505, 2.508, 2.51, 2.511, 2.513, 2.517, 2.533, 2.6};
float NiMh[21] = {2.2,2.315, 2.375, 2.41, 2.442, 2.464, 2.485, 2.498, 2.507, 2.515, 2.522,
	2.525, 2.528, 2.529, 2.53, 2.531, 2.532, 2.534, 2.553, 2.58, 2.7};
float Alkaline[21] = {2.0, 2.0, 2.0, 2.052, 2.105, 2.147, 2.19, 2.215, 2.24, 2.26, 2.28,
	2.315, 2.35, 2.39,  2.43, 2.485, 2.54, 2.615, 2.69, 2.82, 2.95};

flmonView::flmonView(){
}
flmonView::~flmonView(){
 if(window != 0)
		delete window;
}

/* open Window */
void flmonView::open(){

	window = WidgetFactory::new_window("Fl-Mon");

  // memory usage
  mem = new Fl_Slider(10,20,140,15,"");
  mem->type(FL_HOR_FILL_SLIDER);
  mem->color(FL_RED);
	mem->value(0);
	mem->set_output();

  memtext = new Fl_Output(60,3,50,17,"Mem (%):");
	memtext->box(FL_FLAT_BOX);
	memtext->labelsize(10);
	memtext->textsize(10);
	memtext->color(FL_WHITE);
	memtext->value("");

  // cpu usage
  cpu = new Fl_Slider(10,57,140,15,"");
  cpu->type(FL_HOR_FILL_SLIDER);
	cpu->color(FL_RED);
  cpu->value(0);
  cpu->set_output();

  cputext = new Fl_Output(60,40,50,17,"CPU (%):");
	cputext->labelsize(10);
	cputext->textsize(10);
	cputext->color(FL_WHITE);
	cputext->box(FL_FLAT_BOX);
	cputext->value("0");

  // flash usage
  flash = new Fl_Slider(10,97,140,15,"");
  flash->type(FL_HOR_FILL_SLIDER);
	flash->color(FL_RED);
  flash->value(0);
  flash->set_output();

  flashtext = new Fl_Output(60,80,50,17,"Flash (%):");
	flashtext->labelsize(10);
	flashtext->textsize(10);
	flashtext->color(FL_WHITE);
	flashtext->box(FL_FLAT_BOX);
	flashtext->value("0");


  // battery voltage
  bat = new Fl_Slider(10,142,140,15,"");
  bat->type(FL_HOR_FILL_SLIDER);
	bat->color(FL_RED);
  bat->value(0);
  bat->set_output();

  battext = new Fl_Output(68,157,50,17,"Battery (V):");
	battext->labelsize(10);
	battext->textsize(10);
	battext->color(FL_WHITE);
	battext->box(FL_FLAT_BOX);
	battext->value("0");

	battype = new Fl_Choice(47,122,70,15,"Battery");
	battype->labelsize(10);
	battype->textsize(10);
	battype->color(FL_WHITE);
	battype->add("Alkaline",0,0,(void *)"Alkaline");
	battype->add("NiCd",0,0,(void *)"NiCd");
	battype->add("NiMh",0,0,(void*)"NiMh");
	battype->value(readBattype());

  uptime = new Fl_Output(10,185,140,17,"");
	uptime->textsize(12);
	uptime->textfont(FL_HELVETICA_BOLD);
	uptime->color(FL_WHITE);
	uptime->box(FL_THIN_UP_BOX);
	uptime->value("0");


	tools();
	window->end();
	window->show();

	refresh_cb(0,this);

	Fl::add_timeout(FREQU,timer_cb,this);

}

/* toolbar */
void flmonView::tools(){
	Fl_Dockable_Window* toolbar = WidgetFactory::new_toolbar();

	WidgetFactory::new_button("Done", done_cb, this);

	Fl_Button * refresh = WidgetFactory::new_button("Refresh", refresh_cb, this);
	refresh->resize(refresh->x(),refresh->y(),43,refresh->h());

	Fl_Button * about = WidgetFactory::new_button("About", about_cb, this);

	toolbar->end();
	window->add_dockable(toolbar, 1);
}

/* done */
void flmonView::done_cb(Fl_Widget*, void* data){
		flmonView* flm = static_cast<flmonView*>(data);
		
		// Update the preference file and restart apm-sleep
		int bt = flm->battype->value();
    flm->writeBattype(bt);

		delete flm;	
}

/* about */
void flmonView::about_cb(Fl_Widget*, void* data){
		flmonView* flm = static_cast<flmonView*>(data);

		new myAbout(APPNAME,VERSION);
}

/* size of directory */
float flmonView::getDirUsage(char* dir){

  FILE *output;
	char dummy[255] = "", command[255] = "du -s ";
	int kbs = 0, sr;

	strcat(command, dir);

  output = popen (command, "r");
  if (!output) {
      fprintf (stderr, "Could not run du.\n");
      return 0;
  };

  sr = fscanf (output, "%d %s", &kbs, dummy);
	while (sr != EOF) {
	  sr = fscanf (output, "%d %s", &kbs, dummy);
 	};	

  pclose (output);

  return kbs;

}

/* free mem in % */
float flmonView::getMemStat(){

	int	total, used, free, shared, buffer, cached;
	double  user;
	float pc;
	FILE *proc_stat;
	char dummy[256];

	if((proc_stat=fopen(MEMFILE,"r")) == NULL ) {
	  perror(MEMFILE);
		return 0;

	} else {
		fscanf(proc_stat, "%s %s %s %s %s %s", dummy, dummy, dummy, dummy, dummy, dummy);
		fscanf(proc_stat, "%s %ld %ld %ld %ld %ld %ld", dummy, &total, &used, &free, &shared, &buffer, &cached);

		fclose(proc_stat);

		user = (used - buffer -cached);
		pc = user / total;

		return pc;
	};


}

/* cpu usage in % */
float flmonView::getCpuStat(){

	int	user, nice, sys, idle;
	float pc, all, use;
	FILE *proc_stat;
	char dummy[256];

	if((proc_stat=fopen(STATFILE,"r")) == NULL ) {
	  perror(STATFILE);
		return 0;

	} else {
		fscanf(proc_stat, "%s %ld %ld %ld %ld", dummy, &user, &nice, &sys, &idle);

		fclose(proc_stat);

	  if (old_idle == 0) {
			old_idle = idle;
			old_sys = sys;
			old_user = user;
			old_nice = nice;

			return 0;
	  } else {

		  use = (user - old_user) + (sys - old_sys) + (nice - old_nice);
		  all = use + (idle - old_idle);
		  pc = use / all;

			old_idle = idle;
			old_sys = sys;
			old_user = user;
			old_nice = nice;

			return pc;
		};

};

}

/* get the battery voltage*/
float flmonView::batvoltage(){

  int fd;
  int in;
  unsigned short value[4];

  fd = ::open("/dev/adif", O_RDONLY);
  if (fd == -1){
    perror("Opening /dev/adif");
    return 0;
  }

  /* read all 4 registers even if we only care about the first one */
  in = read(fd, &value, 4*sizeof(short));
  close(fd);
  if (in == -1) {
    perror("Reading in /dev/adif");
    return 0;
  }

  return 3.3 * (float) value[0] / 1024.0;

}

/* get battery usage in % */
float flmonView::battlvl(float volts, int battype){

	float * curve;
	int pc = 0, i, n = 21;

	if (battype == 0)
		curve = Alkaline;
	if (battype == 1)
		curve = NiCd;
	if (battype == 2)
		curve = NiMh;
	
	for (i = 0;i < 21; i ++) {
  	if (volts >= curve[i])
			   pc = i;
	};

	return (float) pc / 20.0;		

}

/* timer callback */
void flmonView::timer_cb(void * data){

	flmonView* flm = static_cast<flmonView*>(data);

	flm->updateValues();

 	//Fl::add_timeout((unsigned int)FREQU,timer_cb,flm);
 	Fl::add_timeout(FREQU,timer_cb,flm);

}

/* called by timer */
void flmonView::updateValues(){

	float cpuusage, memory, voltage;
	char buffer[8];

  // get MEM usage
  memory = getMemStat();
  mem->value(memory);
	sprintf(buffer,"%.2f",memory*100);
	memtext->value(buffer);

  // get CPU usage
  cpuusage = getCpuStat();
  cpu->value(cpuusage);
	sprintf(buffer,"%.2f",cpuusage*100);
	cputext->value(buffer);

	// batery voltage
  voltage = batvoltage();
	bat->value(battlvl(voltage,battype->value()));
	sprintf(buffer,"%.3f",voltage);
	battext->value(buffer);

	// uptime
  char up[32] = "";
	getUptime(up);
	uptime->value(up);

}

/* refresh button */
void flmonView::refresh_cb(Fl_Widget*,void* data){
	flmonView* flm = static_cast<flmonView*>(data);

  float kb = flm->getDirUsage(FLASHDIR);
	char buffer[8];

	sprintf(buffer,"%.2f",kb/FLASHSIZE * 100);
	flm->flashtext->value(buffer);
	flm->flash->value(kb/FLASHSIZE);	

}

/* get battype from file */
int flmonView::readBattype(){

		std::map<std::string, std::string> pref;
		char filename[1024];
		sprintf(filename, "%s/.flmon",getenv ("HOME"));
		filename_expand(filename,filename);

		preference_read(filename, &pref);

		sprintf(filename,"%S",pref["battype"].c_str());
		if (pref.count("battype")) {
			return atoi(pref["battype"].c_str());
		};

}

/* write battype to file */
void flmonView::writeBattype(int type){

		std::map<std::string, std::string> pref;
		char filename[1024];
		sprintf(filename, "%s/.flmon",getenv ("HOME"));
		filename_expand(filename,filename);

		char battype[10];
		sprintf (battype, "%d", type);

		pref["battype"] = battype;

		preference_write(filename, &pref);


}

/** get Uptime */
void flmonView::getUptime(char * result){
	struct sysinfo info;
	int updays, uphrs, upmin;

	sysinfo(&info);

	updays = (int) info.uptime / (60*60*24);
	upmin = (int) info.uptime / 60;
	uphrs = (upmin / 60) % 24;
	upmin %= 60;

	snprintf(result, 20, "Uptime %i Days %02i:%02i", updays, uphrs, upmin);

}
